@@ -19,6 +19,8 @@
 //! [Click here for Zenoh's documentation](https://docs.rs/zenoh/latest/zenoh)
 //!
 //! Configuration to pass to `zenoh::open()` and `zenoh::scout()` functions and associated constants.
+#![allow(deprecated)]
+
 pub mod defaults;
 mod include;
 pub mod qos;
@@ -28,12 +30,20 @@ pub mod wrappers;
 use std::convert::TryFrom;
 // This is a false positive from the rust analyser
 use std::{
-    any::Any, collections::HashSet, fmt, io::Read, net::SocketAddr, ops, path::Path, sync::Weak,
+    any::Any,
+    collections::HashSet,
+    fmt,
+    io::Read,
+    net::SocketAddr,
+    num::{NonZeroU16, NonZeroUsize},
+    ops::{self, Bound, Deref, RangeBounds},
+    path::Path,
+    sync::{Arc, Weak},
 };
 
 use include::recursive_include;
 use nonempty_collections::NEVec;
-use qos::{PublisherQoSConfList, QosOverwriteMessage, QosOverwrites};
+use qos::{PublisherQoSConfList, QosFilter, QosOverwriteMessage, QosOverwrites};
 use secrecy::{CloneableSecret, DebugSecret, Secret, SerializableSecret, Zeroize};
 use serde::{Deserialize, Serialize};
 use serde_json::{Map, Value};
@@ -82,8 +92,16 @@ impl Zeroize for SecretString {
 
 pub type SecretValue = Secret<SecretString>;
 
+#[derive(Debug, Deserialize, Serialize, Clone)]
+pub struct TransportWeight {
+    /// A zid of destination node.
+    pub dst_zid: ZenohId,
+    /// A weight of link from this node to the destination.
+    pub weight: NonZeroU16,
+}
+
 #[derive(Debug, Deserialize, Serialize, Clone, Copy, Eq, PartialEq)]
-#[serde(rename_all = "lowercase")]
+#[serde(rename_all = "snake_case")]
 pub enum InterceptorFlow {
     Egress,
     Ingress,
@@ -92,7 +110,10 @@ pub enum InterceptorFlow {
 #[derive(Clone, Copy, Debug, Serialize, Deserialize, Eq, PartialEq)]
 #[serde(rename_all = "snake_case")]
 pub enum DownsamplingMessage {
+    Delete,
+    #[deprecated = "Use `Put` or `Delete` instead."]
     Push,
+    Put,
     Query,
     Reply,
 }
@@ -126,6 +147,19 @@ pub struct DownsamplingItemConf {
     pub flows: Option<NEVec<InterceptorFlow>>,
 }
 
+fn downsampling_validator(d: &Vec<DownsamplingItemConf>) -> bool {
+    for item in d {
+        if item
+            .messages
+            .iter()
+            .any(|m| *m == DownsamplingMessage::Push)
+        {
+            tracing::warn!("In 'downsampling/messages' configuration: 'push' is deprecated and may not be supported in future versions, use 'put' and/or 'delete' instead");
+        }
+    }
+    true
+}
+
 #[derive(Serialize, Debug, Deserialize, Clone)]
 #[serde(deny_unknown_fields)]
 pub struct LowPassFilterConf {
@@ -165,6 +199,81 @@ pub struct AclConfigSubjects {
     pub cert_common_names: Option<NEVec<CertCommonName>>,
     pub usernames: Option<NEVec<Username>>,
     pub link_protocols: Option<NEVec<InterceptorLink>>,
+    pub zids: Option<NEVec<ZenohId>>,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct ConfRange {
+    start: Option<u64>,
+    end: Option<u64>,
+}
+
+impl ConfRange {
+    pub fn new(start: Option<u64>, end: Option<u64>) -> Self {
+        Self { start, end }
+    }
+}
+
+impl RangeBounds<u64> for ConfRange {
+    fn start_bound(&self) -> Bound<&u64> {
+        match self.start {
+            Some(ref start) => Bound::Included(start),
+            None => Bound::Unbounded,
+        }
+    }
+    fn end_bound(&self) -> Bound<&u64> {
+        match self.end {
+            Some(ref end) => Bound::Included(end),
+            None => Bound::Unbounded,
+        }
+    }
+}
+
+impl serde::Serialize for ConfRange {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        serializer.serialize_str(&format!(
+            "{}..{}",
+            self.start.unwrap_or_default(),
+            self.end.unwrap_or_default()
+        ))
+    }
+}
+
+impl<'a> serde::Deserialize<'a> for ConfRange {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'a>,
+    {
+        struct V;
+
+        impl<'de> serde::de::Visitor<'de> for V {
+            type Value = ConfRange;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                formatter.write_str("range string")
+            }
+
+            fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
+            where
+                E: serde::de::Error,
+            {
+                let (start, end) = v
+                    .split_once("..")
+                    .ok_or_else(|| serde::de::Error::custom("invalid range"))?;
+                let parse_bound = |bound: &str| {
+                    (!bound.is_empty())
+                        .then(|| bound.parse::<u64>())
+                        .transpose()
+                        .map_err(|_| serde::de::Error::custom("invalid range bound"))
+                };
+                Ok(ConfRange::new(parse_bound(start)?, parse_bound(end)?))
+            }
+        }
+        deserializer.deserialize_str(V)
+    }
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone)]
@@ -183,11 +292,15 @@ pub struct QosOverwriteItemConf {
     /// List of message types on which the qos overwrite will be applied.
     pub messages: NEVec<QosOverwriteMessage>,
     /// List of key expressions to apply qos overwrite.
-    pub key_exprs: Vec<OwnedKeyExpr>,
+    pub key_exprs: Option<NEVec<OwnedKeyExpr>>,
     // The qos value to overwrite with.
     pub overwrite: QosOverwrites,
     /// QosOverwrite flow directions: egress and/or ingress.
     pub flows: Option<NEVec<InterceptorFlow>>,
+    /// QoS filter to apply to the messages matching this item.
+    pub qos: Option<QosFilter>,
+    /// payload_size range for the messages matching this item.
+    pub payload_size: Option<ConfRange>,
 }
 
 #[derive(Serialize, Debug, Deserialize, Clone, PartialEq, Eq, Hash)]
@@ -233,7 +346,7 @@ pub enum InterceptorLink {
 
 impl std::fmt::Display for InterceptorLink {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "Transport({:?})", self)
+        write!(f, "Transport({self:?})")
     }
 }
 
@@ -270,7 +383,7 @@ pub enum AclMessage {
 }
 
 #[derive(Clone, Copy, Debug, Serialize, Deserialize, Eq, Hash, PartialEq)]
-#[serde(rename_all = "lowercase")]
+#[serde(rename_all = "snake_case")]
 pub enum Permission {
     Allow,
     Deny,
@@ -348,7 +461,8 @@ validated_struct::validator! {
     #[doc(hidden)]
     Config {
         /// The Zenoh ID of the instance. This ID MUST be unique throughout your Zenoh infrastructure and cannot exceed 16 bytes of length. If left unset, a random u128 will be generated.
-        id: ZenohId,
+        /// If not specified a random Zenoh ID will be generated upon session creation.
+        id: Option<ZenohId>,
         /// The metadata of the instance. Arbitrary json data available from the admin space
         metadata: Value,
         /// The node's mode ("router" (default value in `zenohd`), "peer" or "client").
@@ -458,12 +572,24 @@ validated_struct::validator! {
                 /// connected to each other.
                 /// The failover brokering only works if gossip discovery is enabled.
                 peers_failover_brokering: Option<bool>,
+                /// Linkstate mode configuration.
+                pub linkstate: #[derive(Default)]
+                LinkstateConf {
+                    /// Weights of the outgoing links in linkstate mode.
+                    /// If none of the two endpoint nodes of a transport specifies its weight, a weight of 100 is applied.
+                    /// If only one of the two endpoint nodes of a transport specifies its weight, the specified weight is applied.
+                    /// If both endpoint nodes of a transport specify its weight, the greater weight is applied.
+                    pub transport_weights: Vec<TransportWeight>,
+                },
             },
             /// The routing strategy to use in peers and it's configuration.
             pub peer: #[derive(Default)]
             PeerRoutingConf {
                 /// The routing strategy to use in peers. ("peer_to_peer" or "linkstate").
+                /// This option needs to be set to the same value in all peers and routers of the subsystem.
                 mode: Option<String>,
+                /// Linkstate mode configuration (only taken into account if mode == "linkstate").
+                pub linkstate: LinkstateConf,
             },
             /// The interests-based routing configuration.
             /// This configuration applies regardless of the mode (router, peer or client).
@@ -504,6 +630,9 @@ validated_struct::validator! {
                 /// Maximum number of unicast sessions (default: 1000)
                 max_sessions: usize,
                 /// Maximum number of unicast incoming links per transport session (default: 1)
+                /// If set to a value greater than 1, multiple outgoing links are also allowed;
+                /// otherwise, only one outgoing link is allowed.
+                /// Issue https://github.com/eclipse-zenoh/zenoh/issues/1533
                 max_links: usize,
                 /// Enables the LowLatency transport (default `false`).
                 /// This option does not make LowLatency transport mandatory, the actual implementation of transport
@@ -674,6 +803,16 @@ validated_struct::validator! {
                 /// - "init": SHM subsystem internals will be initialized upon Session opening. This setting sacrifices
                 /// startup time, but guarantees no latency impact when first SHM buffer is processed.
                 mode: ShmInitMode,
+                pub transport_optimization:
+                LargeMessageTransportOpt {
+                    /// Enables transport optimization for large messages (default `true`).
+                    /// Implicitly puts large messages into shared memory for transports with SHM-compatible connection.
+                    enabled: bool,
+                    /// SHM arena size in bytes used for transport optimization (default `16 * 1024 * 1024`).
+                    pool_size: NonZeroUsize,
+                    /// Allow optimization for messages equal or larger than this threshold in bytes (default `3072`).
+                    message_size_threshold: usize,
+                },
             },
             pub auth: #[derive(Default)]
             AuthConf {
@@ -733,9 +872,9 @@ validated_struct::validator! {
         pub namespace: Option<OwnedNonWildKeyExpr>,
 
         /// Configuration of the downsampling.
-        downsampling: Vec<DownsamplingItemConf>,
+        downsampling: Vec<DownsamplingItemConf> where (downsampling_validator),
 
-        ///Configuration of the access control (ACL)
+        /// Configuration of the access control (ACL)
         pub access_control: AclConfig {
             pub enabled: bool,
             pub default_permission: Permission,
@@ -871,6 +1010,183 @@ fn config_deser() {
             .unwrap(),
     )
         .unwrap_err());
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    overwrite: {
+                      priority: "foo",
+                    },
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    );
+    assert!(config.is_err());
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    overwrite: {
+                      priority: +8,
+                    },
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    );
+    assert!(config.is_err());
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    overwrite: {
+                      priority: "data_high",
+                    },
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    )
+    .unwrap();
+    assert_eq!(
+        config.qos().network().first().unwrap().overwrite.priority,
+        Some(qos::PriorityUpdateConf::Priority(
+            qos::PriorityConf::DataHigh
+        ))
+    );
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    overwrite: {
+                      priority: +1,
+                    },
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    )
+    .unwrap();
+    assert_eq!(
+        config.qos().network().first().unwrap().overwrite.priority,
+        Some(qos::PriorityUpdateConf::Increment(1))
+    );
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    payload_size: "0..99",
+                    overwrite: {},
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    )
+    .unwrap();
+    assert_eq!(
+        config
+            .qos()
+            .network()
+            .first()
+            .unwrap()
+            .payload_size
+            .as_ref()
+            .map(|r| (r.start_bound(), r.end_bound())),
+        Some((Bound::Included(&0), Bound::Included(&99)))
+    );
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    payload_size: "100..",
+                    overwrite: {},
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    )
+    .unwrap();
+    assert_eq!(
+        config
+            .qos()
+            .network()
+            .first()
+            .unwrap()
+            .payload_size
+            .as_ref()
+            .map(|r| (r.start_bound(), r.end_bound())),
+        Some((Bound::Included(&100), Bound::Unbounded))
+    );
+
+    let config = Config::from_deserializer(
+        &mut json5::Deserializer::from_str(
+            r#"{
+              qos: {
+                network: [
+                  {
+                    messages: ["put"],
+                    qos: {
+                      congestion_control: "drop",
+                      priority: "data",
+                      express: true,
+                      reliability: "reliable",
+                    },
+                    overwrite: {},
+                  },
+                ],
+              }
+            }"#,
+        )
+        .unwrap(),
+    )
+    .unwrap();
+    assert_eq!(
+        config.qos().network().first().unwrap().qos,
+        Some(QosFilter {
+            congestion_control: Some(qos::CongestionControlConf::Drop),
+            priority: Some(qos::PriorityConf::Data),
+            express: Some(true),
+            reliability: Some(qos::ReliabilityConf::Reliable),
+        })
+    );
+
     dbg!(Config::from_file("../../DEFAULT_CONFIG.json5").unwrap());
 }
 
@@ -889,7 +1205,7 @@ impl Config {
     pub fn get(
         &self,
         key: &str,
-    ) -> Result<<Self as ValidatedMapAssociatedTypes>::Accessor, GetError> {
+    ) -> Result<<Self as ValidatedMapAssociatedTypes<'_>>::Accessor, GetError> {
         <Self as ValidatedMap>::get(self, key)
     }
 
@@ -979,6 +1295,9 @@ impl Config {
                 if let Err(e) = f.read_to_string(&mut content) {
                     bail!(e)
                 }
+                if content.is_empty() {
+                    bail!("Empty config file");
+                }
                 match path
                     .extension()
                     .map(|s| s.to_str().unwrap())
@@ -1140,7 +1459,7 @@ impl PluginsConfig {
             let required = match value.get("__required__") {
                 None => false,
                 Some(Value::Bool(b)) => *b,
-                _ => panic!("Plugin '{}' has an invalid '__required__' configuration property (must be a boolean)", id)
+                _ => panic!("Plugin '{id}' has an invalid '__required__' configuration property (must be a boolean)")
             };
             let name = match value.get("__plugin__") {
                 Some(Value::String(p)) => p,
@@ -1150,8 +1469,8 @@ impl PluginsConfig {
             if let Some(paths) = value.get("__path__") {
                 let paths = match paths {
                     Value::String(s) => vec![s.clone()],
-                    Value::Array(a) => a.iter().map(|s| if let Value::String(s) = s { s.clone() } else { panic!("Plugin '{}' has an invalid '__path__' configuration property (must be either string or array of strings)", id) }).collect(),
-                    _ => panic!("Plugin '{}' has an invalid '__path__' configuration property (must be either string or array of strings)", id)
+                    Value::Array(a) => a.iter().map(|s| if let Value::String(s) = s { s.clone() } else { panic!("Plugin '{id}' has an invalid '__path__' configuration property (must be either string or array of strings)") }).collect(),
+                    _ => panic!("Plugin '{id}' has an invalid '__path__' configuration property (must be either string or array of strings)")
                 };
                 PluginLoad { id: id.clone(), name: name.clone(), paths: Some(paths), required }
             } else {
@@ -1264,7 +1583,7 @@ impl std::fmt::Debug for PluginsConfig {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let mut values: Value = self.values.clone();
         sift_privates(&mut values);
-        write!(f, "{:?}", values)
+        write!(f, "{values:?}")
     }
 }
 
@@ -1450,3 +1769,43 @@ macro_rules! unwrap_or_default {
         $val$(.$field($($param)?))*.clone().unwrap_or(zenoh_config::defaults$(::$field$(($param))?)*.into())
     };
 }
+
+pub trait IConfig: Send + Sync {
+    fn get(&self, key: &str) -> ZResult<String>;
+    fn queries_default_timeout_ms(&self) -> u64;
+    fn insert_json5(&self, key: &str, value: &str) -> ZResult<()>;
+    fn to_json(&self) -> String;
+}
+
+pub struct GenericConfig(Arc<dyn IConfig>);
+
+impl Deref for GenericConfig {
+    type Target = Arc<dyn IConfig>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl GenericConfig {
+    pub fn new(value: Arc<dyn IConfig>) -> Self {
+        GenericConfig(value)
+    }
+
+    pub fn get_typed<T: for<'a> Deserialize<'a>>(&self, key: &str) -> ZResult<T> {
+        self.0
+            .get(key)
+            .and_then(|v| serde_json::from_str::<T>(&v).map_err(|e| e.into()))
+    }
+
+    pub fn get_plugin_config(&self, plugin_name: &str) -> ZResult<Value> {
+        self.get(&("plugins/".to_owned() + plugin_name))
+            .and_then(|v| serde_json::from_str(&v).map_err(|e| e.into()))
+    }
+}
+
+impl fmt::Display for GenericConfig {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(&self.0.to_json())
+    }
+}
